#######################################################
# Funktion zum Normalisieren einer Rufnummer
# Version 1.1 22.09.2019 Alexander Kromat
# bulletproof-admin.de
#
# Der Code kann gerne verwendet werden, aber bitte den Kopf drin lassen
#######################################################

# Quellen sind die Bundesnetzagentur

# Festnetz-Vorwahlen
# Download der Vorwahlen unter https://www.bundesnetzagentur.de/SharedDocs/Downloads/DE/Sachgebiete/Telekommunikation/Unternehmen_Institutionen/Nummerierung/Rufnummern/ONRufnr/Vorwahlverzeichnis_ONBzip.html
# Dann auspacken und in den Ordner legen

# Mobilfunk Vorwahlen
# Download unter https://www.bundesnetzagentur.de/DE/Sachgebiete/Telekommunikation/Unternehmen_Institutionen/Nummerierung/Rufnummern/MobileDienste/zugeteilte%20RNB/MobileDiensteBelegteRNB_Basepage.html

# Damit es funktioniert werden die 3 CSV Dateien mit den Vorwahltypen im aktuellen Ordner benötigt
# Die Funktion wird dann aufgerufen mit 
# Format_Phonenumber -Phonenumer "030-1234567"
# Als Return kommt die formatierte Nummer oder $false, wenn es keinen Match gab (z.B. Vorwahl falsch)
# Die testnummern.txt ist für die Funktion nötig sondern dienst hier als Testliste
###################################################################################################################

# Funktionen
function Format_Phonenumber 
    {Param (
        [parameter(Mandatory=$true,ParameterSetName="Phonenumber")]
        [string]$Phonenumber)
    
    # Alle Sonderzeichen entfernen per RegEx
$number = $Phonenumber -replace "[^0-9]"

#Führende 49 entfernen
#$number = $number.TrimStart("49")
if ($number.substring(0,2)-eq "49")
    {
    $number = $number.substring(2)
    }
    
#Führende Null entfernen
$number = $number.TrimStart("0")

# Ab hier wird die passende Vorwahl gesucht

for ($i=5; $i -gt 1; $i=$i-1) 
{
    # Die Vorwahl wird Stelle für Stelle gesucht, bis genau 1 x passt
    $compare = $number.substring(0,$i)
    $treffer = ($vorwahlen.Ortsnetzkennzahl.substring(0,$i)) -match $compare
    if ($treffer.count -eq 1)
        {
        # Schleife abbrechen wenn nur 1 Ergebniss (das passende)
        break
        }
}
if ($i -gt 1)
    {
    # Jetzt die Nummer formatieren weil gefunden
    # "Treffer gefunden"
    $number = "+49 "+ $number.substring(0,$i) + " " +$number.substring($i)
    }
    else
    {
    # "Vorwahl falsch / Nicht gefunden"
    # Wir geben den Input zurück... weil nix besseres gefunden
    return $false
    }

    return $number
}

#######################################################################
# Variablen
# Diese werden in der Funktion benötigt und müssen daher einmal durchlaufen werden
$vorwahlen = Import-Csv -Path .\festnetz.csv -Delimiter ";"
$mobilfunk = Import-Csv -Path .\mobilfunk.csv -Delimiter ";"
$sondernummern = Import-Csv -Path .\sondernummern.csv -Delimiter ";"

# Liste zusammenfügen
$vorwahlen = $vorwahlen + $mobilfunk + $sondernummern
 
# Ein paar Leerstellen anhängen...Sonst fehler beim Match Befehl
$vorwahlen | ForEach-Object{ $_.Ortsnetzkennzahl = $_.Ortsnetzkennzahl + "        " }

##########################################################################################
# Hier könnte dann euer Code kommen oder eben das Beispiel:

##############################################################################################################################################
# Ab hier testen wir etwas mit einer Liste von Nummern. Das ganze kann auch z.B. im Rahmen einer SQL Abfrage oder einer Eingabemaske laufen
$testzahlen = Get-Content -Path .\testnummern.txt

foreach ($nummer in $testzahlen) 
    {
        $newnumber = Format_Phonenumber -Phonenumber $nummer
        if ($newnumber)
        {
        write-host "Konvertiere"  $nummer  "und erhalte" $newnumber -ForegroundColor Green
        }
        else
        {
        write-host "Fehlerhaft:" $nummer  "nicht in Vorwahlenliste gefunden" -ForegroundColor Red
        }
    }
    
